@@ -2,7 +2,7 @@
 #'
 #' Create a covariance matrix from a fitted model object.
 #'
-#' @param object A fitted model object from [splm()] or [spautor()].
+#' @param object A fitted model object (e.g., [splm()], [spautor()], [spglm()], or [spgautor()]).
 #' @param newdata If omitted, the covariance matrix of
 #'   the observed data is returned. If provided, \code{newdata} is
 #'   a data frame or \code{sf} object that contains coordinate information
@@ -17,8 +17,9 @@
 #'   data, which has dimension n x n, where n is the sample size used to fit \code{object}.
 #'   If \code{newdata} is provided, the covariance matrix between the unobserved (new)
 #'   data and the observed data, which has dimension m x n, where m is the number of
-#'   new observations and n is the samle size used to fit \code{object}.
+#'   new observations and n is the sample size used to fit \code{object}.
 #'
+#' @order 1
 #' @export
 #'
 #' @examples
@@ -30,7 +31,9 @@
 covmatrix <- function(object, newdata, ...) {
   UseMethod("covmatrix", object)
 }
+#' @rdname covmatrix
 #' @method covmatrix splm
+#' @order 2
 #' @export
 covmatrix.splm <- function(object, newdata, ...) {
 
@@ -43,8 +46,10 @@ covmatrix.splm <- function(object, newdata, ...) {
   if (missing(newdata)) {
     # coordinate stuff
     if (object$anisotropy) {
-      new_coords <- transform_anis(object$obdata, object$xcoord, object$ycoord,
-                                   spcov_params_val[["rotate"]], spcov_params_val[["scale"]])
+      new_coords <- transform_anis(
+        object$obdata, object$xcoord, object$ycoord,
+        spcov_params_val[["rotate"]], spcov_params_val[["scale"]]
+      )
       dist_matrix <- spdist(xcoord_val = new_coords$xcoord_val, ycoord_val = new_coords$ycoord_val)
     } else {
       dist_matrix <- spdist(object$obdata, object$xcoord, object$ycoord)
@@ -60,8 +65,10 @@ covmatrix.splm <- function(object, newdata, ...) {
 
     # cov matrix
     cov_val <- cov_matrix(spcov_params_val, dist_matrix, randcov_params_val,
-                          randcov_Zs_val, partition_matrix = partition_matrix_val)
-
+      randcov_Zs_val,
+      partition_matrix = partition_matrix_val,
+      diagtol = object$diagtol
+    )
   } else {
     # rename relevant quantities
     obdata <- object$obdata
@@ -86,14 +93,14 @@ covmatrix.splm <- function(object, newdata, ...) {
 
     if (object$anisotropy) { # could just do rotate != 0 || scale != 1
       obdata_aniscoords <- transform_anis(obdata, xcoord, ycoord,
-                                          rotate = spcov_params_val[["rotate"]],
-                                          scale = spcov_params_val[["scale"]]
+        rotate = spcov_params_val[["rotate"]],
+        scale = spcov_params_val[["scale"]]
       )
       obdata[[xcoord]] <- obdata_aniscoords$xcoord_val
       obdata[[ycoord]] <- obdata_aniscoords$ycoord_val
       newdata_aniscoords <- transform_anis(newdata, xcoord, ycoord,
-                                           rotate = spcov_params_val[["rotate"]],
-                                           scale = spcov_params_val[["scale"]]
+        rotate = spcov_params_val[["rotate"]],
+        scale = spcov_params_val[["scale"]]
       )
       newdata[[xcoord]] <- newdata_aniscoords$xcoord_val
       newdata[[ycoord]] <- newdata_aniscoords$ycoord_val
@@ -114,10 +121,11 @@ covmatrix.splm <- function(object, newdata, ...) {
 
   # return covariance value as a base R matrix (not a Matrix matrix)
   as.matrix(cov_val)
-
 }
 
+#' @rdname covmatrix
 #' @method covmatrix spautor
+#' @order 3
 #' @export
 covmatrix.spautor <- function(object, newdata, ...) {
 
@@ -140,7 +148,9 @@ covmatrix.spautor <- function(object, newdata, ...) {
 
   # full cov matrix
   cov_matrix_val <- cov_matrix(spcov_params_val, dist_matrix, randcov_params_val,
-                               randcov_Zs_val, partition_matrix = partition_matrix_val, object$M)
+    randcov_Zs_val,
+    partition_matrix = partition_matrix_val, object$M
+  )
 
   if (missing(newdata)) {
     # observed covariance matrix
@@ -152,5 +162,4 @@ covmatrix.spautor <- function(object, newdata, ...) {
 
   # return covariance value as a base R matrix (not a Matrix matrix)
   as.matrix(cov_val)
-
 }
